from __future__ import annotations as _annotations

from typing import Any

from pydantic import AwareDatetime, BaseModel
from pydantic_evals.dataset import Dataset
from pydantic_evals.evaluators.common import IsInstance, LLMJudge
from typing_extensions import TypedDict


class TimeRangeBuilderSuccess(BaseModel, use_attribute_docstrings=True):
    """Response when a time range could be successfully generated."""

    min_timestamp_with_offset: AwareDatetime
    """A datetime in ISO format with timezone offset."""

    max_timestamp_with_offset: AwareDatetime
    """A datetime in ISO format with timezone offset."""

    explanation: str | None
    """
    A brief explanation of the time range that was selected.

    For example, if a user only mentions a specific point in time, you might explain that you selected a 10 minute
    window around that time.
    """

    def __str__(self):
        readable_min_timestamp = self.min_timestamp_with_offset.strftime(
            '%A, %B %d, %Y %H:%M:%S %Z'
        )
        readable_max_timestamp = self.max_timestamp_with_offset.strftime(
            '%A, %B %d, %Y %H:%M:%S %Z'
        )
        lines = [
            'TimeRangeBuilderSuccess:',
            f'* min_timestamp_with_offset: {readable_min_timestamp}',
            f'* max_timestamp_with_offset: {readable_max_timestamp}',
        ]
        if self.explanation is not None:
            lines.append(f'* explanation: {self.explanation}')
        return '\n'.join(lines)


class TimeRangeBuilderError(BaseModel):
    """Response when a time range cannot not be generated."""

    error_message: str

    def __str__(self):
        return f'TimeRangeBuilderError:\n* {self.error_message}'


TimeRangeResponse = TimeRangeBuilderSuccess | TimeRangeBuilderError


class TimeRangeInputs(TypedDict):
    """The inputs for the time range inference agent."""

    prompt: str
    now: AwareDatetime


# TODO(DavidM): Drop the MetadataT type parameter and use the default below once pydantic 2.11 is in use
class TimeRangeDataset(Dataset[TimeRangeInputs, TimeRangeResponse, dict[str, Any]]):
    """A dataset of examples for the time range inference agent."""

    pass


if __name__ == '__main__':

    def main():
        """Usage example."""
        from pathlib import Path

        path = Path(__file__).parent / 'test_cases.yaml'
        if not path.exists():
            TimeRangeDataset(cases=[], evaluators=[LLMJudge(rubric='abc')]).to_file(
                path, custom_evaluator_types=[LLMJudge, IsInstance]
            )

    main()



import asyncio
from datetime import datetime

import logfire
from logfire import ConsoleOptions

from pydantic_ai_examples.evals import infer_time_range
from pydantic_ai_examples.evals.models import TimeRangeInputs

if __name__ == '__main__':

    async def main():
        """Example usage of the time range inference agent."""
        logfire.configure(
            send_to_logfire='if-token-present', console=ConsoleOptions(verbose=True)
        )
        user_prompt = 'yesterday from 2-4 ET'
        # user_prompt = 'the last 24 hours'
        # user_prompt = '6 to 9 PM ET on October 8th'
        # user_prompt = 'next week'
        # user_prompt = 'what time is it?'

        print(
            await infer_time_range(
                TimeRangeInputs(prompt=user_prompt, now=datetime.now().astimezone())
            )
        )

    asyncio.run(main())



from pydantic_evals.dataset import EvaluatorContext
from pydantic_evals.evaluators import Evaluator
from pydantic_evals.evaluators.common import IsInstance, LLMJudge
from pydantic_evals.evaluators.llm_as_a_judge import GradingOutput, judge_input_output

from .agent import infer_time_range
from .models import TimeRangeDataset, TimeRangeInputs, TimeRangeResponse


async def judge_time_range_case(
    inputs: TimeRangeInputs, output: TimeRangeResponse
) -> GradingOutput:
    """Judge the output of a time range inference agent based on a rubric."""
    rubric = (
        'The output should be a reasonable time range to select for the given inputs, or an error '
        'message if no good time range could be selected. Pick a score between 0 and 1 to represent how confident '
        'you are that the selected time range was what the user intended, or that an error message was '
        'an appropriate response.'
    )
    return await judge_input_output(inputs, output, rubric)


async def main():
    """TODO: Task: Convert this pydantic_evals.demo package into docs."""
    from pathlib import Path

    import logfire

    logfire.configure(
        send_to_logfire='if-token-present', console=logfire.ConsoleOptions(verbose=True)
    )
    dataset = TimeRangeDataset.from_file(
        Path(__file__).parent / 'test_cases.yaml',
        custom_evaluator_types=(IsInstance, LLMJudge),
    )

    class MyEvaluator(Evaluator[TimeRangeInputs, TimeRangeResponse]):
        async def evaluate(
            self, ctx: EvaluatorContext[TimeRangeInputs, TimeRangeResponse]
        ):
            result = await judge_time_range_case(inputs=ctx.inputs, output=ctx.output)
            return {
                'is_reasonable': 'yes' if result.pass_ else 'no',
                'accuracy': result.score,
            }

    dataset.add_evaluator(MyEvaluator())

    report = await dataset.evaluate(infer_time_range, max_concurrency=10)
    report.print(include_input=True, include_output=True)


if __name__ == '__main__':
    import asyncio

    asyncio.run(main())
